/*
  Reading and writing data to 24C08 EEPROM over I2C
  CC0 
  
  Datasheet for 24C08 http://www.datasheetcatalog.org/datasheets/134/80474_DS.pdf
  
  WIRING
            __ __
      PRE -|     |- VCC
      NC  -|     |- MODE
      E   -|     |- SCL
      VSS -|_____|- SDA
          
      PRE  = Write Protect              -> GROUND
      NC   = Not connected       
      E    = Chip Enable Input          -> first EEPROM -> GROUND for I2C addresses 0x50,0x51,0x52,0x53; second EEPROM -> 5V for I2C addresses 0x54,0x55,0x56,0x57
      VSS  = Ground                     -> GROUND
      VCC  = Supply Voltage             -> 5V
      MODE = Multibyte/Page Write Mode  -> GROUND (not used here)
      SCL  = Serial Clock               -> Analog PIN 5 (Uno)
      SDA  = Serial Data Address IN/OUT -> Analog PIN 4 (Uno)
  
  
*/

#include <Wire.h>     // for I2C

// variable to receive data coming from the EERPROMs
byte data_received=0; 

// eeprom address array, use I2C scanner sketch to find the addresses. This array contains 2 chips with 4 device addresses (like virtual chips) each
byte chip[] = {0x54,0x55,0x56,0x57}; 



void setup()
{
  Serial.begin(9600); // for screen output
  Wire.begin();       // waking up I2C
  delay(1000);
  Serial.println("Setup finished");
}

void writeData(int device_address, byte memory_address, byte data) 
// write one byte of data 'data' to eeprom memory address 'memory_address' to chip with I2C address 'device_address'
{
  Wire.beginTransmission(device_address);  // device address
  Wire.write(memory_address );             // memory address
  Wire.write(data);                        // data to send
  Wire.endTransmission();                  // end
  delay(10);
}

byte readData(int device_address, byte memory_address) 
// reads one byte of data from memory location 'memory_address' in chip at I2C address 'device_address' 
{
  byte result;  // return value
  Wire.beginTransmission(device_address); // device address
  Wire.write(memory_address);             // memory address
  Wire.endTransmission();                 // end
  Wire.requestFrom(device_address,1);     // get one byte of data from device
  if (Wire.available()) 
    result = Wire.read();
  return result;                          // return the read data byte
  delay(10);
}

void loop()
{
  delay(1000);                                              // let's wait some seconds
  
  // READING DATA
    
  Serial.println("Reading data...");
  for (int a=0; a<4; a++)                                    // same as above, we toogle through the 8 chips ....
  {
    Serial.print("Chip: ");
    Serial.println(chip[a],HEX);
    for (int b=0; b<256;b++)                                 // and all 256 addresses in each (virtual) chip...
    {
      data_received=readData(chip[a],b);                     // and read the data there....
      if(data_received < 16) Serial.print("0");
      Serial.print(data_received, HEX);                     // and print it to serial....
      Serial.print(" ");
    }
    Serial.println();                                        // new line for each chip
  }
// since the chip has only a limited number of write cycles, we don't want to write again and again. Let's end the sketch
  do                                                         
  {
    delay(10000);
  } while ( 1 != 0);
}
